@@ -46,6 +46,9 @@
 #include "clutter-fixed.h"
 #include "clutter-marshal.h"
 #include "clutter-private.h"
+#include "clutter-profile.h"
+
+#include <cogl/cogl.h>
 
 G_DEFINE_ABSTRACT_TYPE (ClutterBackend, clutter_backend, G_TYPE_OBJECT);
 
@@ -300,10 +303,24 @@ _clutter_backend_redraw (ClutterBackend *backend,
                          ClutterStage   *stage)
 {
   ClutterBackendClass *klass;
+  CLUTTER_STATIC_COUNTER (redraw_counter,
+                          "_clutter_backend_redraw counter",
+                          "Increments for each _clutter_backend_redraw call",
+                          0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (redraw_timer,
+                        "Mainloop", /* parent */
+                        "Redrawing",
+                        "The time spent redrawing everything",
+                        0 /* no application private data */);
+
+  CLUTTER_COUNTER_INC (_clutter_uprof_context, redraw_counter);
+  CLUTTER_TIMER_START (_clutter_uprof_context, redraw_timer);
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
   if (G_LIKELY (klass->redraw))
     klass->redraw (backend, stage);
+
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, redraw_timer);
 }
 
 gboolean
@@ -321,12 +338,20 @@ _clutter_backend_create_context (ClutterBackend  *backend,
   return TRUE;
 }
 
+void
+_clutter_backend_ensure_context_internal (ClutterBackend  *backend,
+                                          ClutterStage    *stage)
+{
+  ClutterBackendClass *klass = CLUTTER_BACKEND_GET_CLASS (backend);
+  if (G_LIKELY (klass->ensure_context))
+    klass->ensure_context (backend, stage);
+}
+
 void
 _clutter_backend_ensure_context (ClutterBackend *backend,
                                  ClutterStage   *stage)
 {
   static ClutterStage *current_context_stage = NULL;
-  ClutterBackendClass *klass;
 
   g_return_if_fail (CLUTTER_IS_BACKEND (backend));
   g_return_if_fail (CLUTTER_IS_STAGE (stage));
@@ -352,10 +377,23 @@ _clutter_backend_ensure_context (ClutterBackend *backend,
                         new_stage);
         }
 
-      klass = CLUTTER_BACKEND_GET_CLASS (backend);
-      if (G_LIKELY (klass->ensure_context))
-        klass->ensure_context (backend, new_stage);
-      
+      _clutter_backend_ensure_context_internal (backend, new_stage);
+
+      /* XXX: Until Cogl becomes fully responsible for backend windows
+       * Clutter need to manually keep it informed of the current window size
+       *
+       * NB: This must be done after we ensure_context above because Cogl
+       * always assumes there is a current GL context.
+       */
+      if (new_stage)
+        {
+          float width, height;
+
+          clutter_actor_get_size (CLUTTER_ACTOR (stage), &width, &height);
+
+          _cogl_onscreen_clutter_backend_set_size (width, height);
+        }
+
       /* FIXME: With a NULL stage and thus no active context it may make more
        * sense to clean the context but then re call with the default stage 
        * so at least there is some kind of context in place (as to avoid
@@ -387,6 +425,14 @@ _clutter_backend_get_features (ClutterBackend *backend)
   g_return_val_if_fail (CLUTTER_IS_BACKEND (backend), 0);
 
   klass = CLUTTER_BACKEND_GET_CLASS (backend);
+
+  /* we need to have a context here; so we create the
+   * GL context first and the ask for features. if the
+   * context already exists this should be a no-op
+   */
+  if (klass->create_context)
+    klass->create_context (backend, NULL);
+
   if (klass->get_features)
     return klass->get_features (backend);
   
