@@ -15,9 +15,6 @@ GType clutter_actor_flags_get_type (void) G_GNUC_CONST;
 GType clutter_allocation_flags_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_ALLOCATION_FLAGS (clutter_allocation_flags_get_type())
 
-GType clutter_redraw_flags_get_type (void) G_GNUC_CONST;
-#define CLUTTER_TYPE_REDRAW_FLAGS (clutter_redraw_flags_get_type())
-
 /* enumerations from "./clutter-animator.h" */
 GType clutter_interpolation_get_type (void) G_GNUC_CONST;
 #define CLUTTER_TYPE_INTERPOLATION (clutter_interpolation_get_type())
