@@ -109,6 +109,7 @@
 #include "clutter-debug.h"
 #include "clutter-version.h" 	/* For flavour define */
 #include "clutter-frame-source.h"
+#include "clutter-profile.h"
 
 #include "cogl/cogl.h"
 #include "pango/cogl-pango.h"
@@ -136,6 +137,7 @@ static guint clutter_main_loop_level         = 0;
 static GSList *main_loops                    = NULL;
 
 guint clutter_debug_flags = 0;  /* global clutter debug flag */
+guint clutter_profile_flags = 0;  /* global clutter profile flag */
 
 const guint clutter_major_version = CLUTTER_MAJOR_VERSION;
 const guint clutter_minor_version = CLUTTER_MINOR_VERSION;
@@ -164,6 +166,13 @@ static const GDebugKey clutter_debug_keys[] = {
 };
 #endif /* CLUTTER_ENABLE_DEBUG */
 
+#ifdef CLUTTER_ENABLE_PROFILE
+static const GDebugKey clutter_profile_keys[] = {
+      {"picking-only", CLUTTER_PROFILE_PICKING_ONLY },
+      {"disable-report", CLUTTER_PROFILE_DISABLE_REPORT }
+};
+#endif /* CLUTTER_ENABLE_DEBUG */
+
 /**
  * clutter_get_show_fps:
  *
@@ -187,10 +196,16 @@ _clutter_stage_maybe_relayout (ClutterActor *stage)
 {
   gfloat natural_width, natural_height;
   ClutterActorBox box = { 0, };
+  CLUTTER_STATIC_TIMER (relayout_timer,
+                        "Mainloop", /* no parent */
+                        "Layouting",
+                        "The time spent reallocating the stage",
+                        0 /* no application private data */);
 
   /* avoid reentrancy */
   if (!(CLUTTER_PRIVATE_FLAGS (stage) & CLUTTER_ACTOR_IN_RELAYOUT))
     {
+      CLUTTER_TIMER_START (_clutter_uprof_context, relayout_timer);
       CLUTTER_NOTE (ACTOR, "Recomputing layout");
 
       CLUTTER_SET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_IN_RELAYOUT);
@@ -212,6 +227,7 @@ _clutter_stage_maybe_relayout (ClutterActor *stage)
       clutter_actor_allocate (stage, &box, CLUTTER_ALLOCATION_NONE);
 
       CLUTTER_UNSET_PRIVATE_FLAGS (stage, CLUTTER_ACTOR_IN_RELAYOUT);
+      CLUTTER_TIMER_STOP (_clutter_uprof_context, relayout_timer);
     }
 }
 
@@ -529,10 +545,44 @@ _clutter_do_pick (ClutterStage   *stage,
   CoglColor           stage_pick_id;
   guint32             id;
   GLboolean           dither_was_on;
+  ClutterActor       *actor;
+  CLUTTER_STATIC_COUNTER (do_pick_counter,
+                          "_clutter_do_pick counter",
+                          "Increments for each full pick run",
+                          0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (pick_timer,
+                        "Mainloop", /* parent */
+                        "Picking",
+                        "The time spent picking",
+                        0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (pick_clear,
+                        "Picking", /* parent */
+                        "Stage clear (pick)",
+                        "The time spent clearing stage for picking",
+                        0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (pick_paint,
+                        "Picking", /* parent */
+                        "Painting actors (pick mode)",
+                        "The time spent painting actors in pick mode",
+                        0 /* no application private data */);
+  CLUTTER_STATIC_TIMER (pick_read,
+                        "Picking", /* parent */
+                        "Read Pixels",
+                        "The time spent issuing a read pixels",
+                        0 /* no application private data */);
+
 
   if (clutter_debug_flags & CLUTTER_DEBUG_NOP_PICKING)
     return CLUTTER_ACTOR (stage);
 
+#ifdef CLUTTER_ENABLE_PROFILE
+  if (clutter_profile_flags & CLUTTER_PROFILE_PICKING_ONLY)
+    _clutter_profile_resume ();
+#endif /* CLUTTER_ENABLE_PROFILE */
+
+  CLUTTER_COUNTER_INC (_clutter_uprof_context, do_pick_counter);
+  CLUTTER_TIMER_START (_clutter_uprof_context, pick_timer);
+
   context = _clutter_context_get_default ();
 
   _clutter_backend_ensure_context (context->backend, stage);
@@ -545,9 +595,11 @@ _clutter_do_pick (ClutterStage   *stage,
 
   cogl_disable_fog ();
   cogl_color_set_from_4ub (&stage_pick_id, 255, 255, 255, 255);
+  CLUTTER_TIMER_START (_clutter_uprof_context, pick_clear);
   cogl_clear (&stage_pick_id,
 	      COGL_BUFFER_BIT_COLOR |
 	      COGL_BUFFER_BIT_DEPTH);
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_clear);
 
   /* Disable dithering (if any) when doing the painting in pick mode */
   dither_was_on = glIsEnabled (GL_DITHER);
@@ -557,9 +609,11 @@ _clutter_do_pick (ClutterStage   *stage,
   /* Render the entire scence in pick mode - just single colored silhouette's
    * are drawn offscreen (as we never swap buffers)
   */
+  CLUTTER_TIMER_START (_clutter_uprof_context, pick_paint);
   context->pick_mode = mode;
   clutter_actor_paint (CLUTTER_ACTOR (stage));
   context->pick_mode = CLUTTER_PICK_NONE;
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_paint);
 
   if (G_LIKELY (!(clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS)))
     cogl_clip_pop ();
@@ -568,10 +622,12 @@ _clutter_do_pick (ClutterStage   *stage,
   cogl_flush ();
 
   /* Read the color of the screen co-ords pixel */
+  CLUTTER_TIMER_START (_clutter_uprof_context, pick_read);
   cogl_read_pixels (x, y, 1, 1,
                     COGL_READ_PIXELS_COLOR_BUFFER,
                     COGL_PIXEL_FORMAT_RGBA_8888,
                     pixel);
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_read);
 
   if (G_UNLIKELY (clutter_debug_flags & CLUTTER_DEBUG_DUMP_PICK_BUFFERS))
     {
@@ -585,11 +641,24 @@ _clutter_do_pick (ClutterStage   *stage,
     glEnable (GL_DITHER);
 
   if (pixel[0] == 0xff && pixel[1] == 0xff && pixel[2] == 0xff)
-    return CLUTTER_ACTOR (stage);
+    {
+      actor = CLUTTER_ACTOR (stage);
+      goto result;
+    }
 
   id = _clutter_pixel_to_id (pixel);
+  actor = clutter_get_actor_by_gid (id);
 
-  return clutter_get_actor_by_gid (id);
+result:
+
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, pick_timer);
+
+#ifdef CLUTTER_ENABLE_PROFILE
+  if (clutter_profile_flags & CLUTTER_PROFILE_PICKING_ONLY)
+    _clutter_profile_suspend ();
+#endif
+
+  return actor;
 }
 
 static ClutterTextDirection
@@ -669,7 +738,7 @@ _clutter_context_get_pango_context (ClutterMainContext *self)
     {
       PangoContext *context;
 
-      context = cogl_pango_font_map_create_context (self->font_map);
+      context = _clutter_context_create_pango_context (self);
       self->pango_context = context;
 
       g_signal_connect (self->backend, "resolution-changed",
@@ -679,8 +748,8 @@ _clutter_context_get_pango_context (ClutterMainContext *self)
                         G_CALLBACK (update_pango_context),
                         self->pango_context);
     }
-
-  update_pango_context (self->backend, self->pango_context);
+  else
+    update_pango_context (self->backend, self->pango_context);
 
   return self->pango_context;
 }
@@ -690,6 +759,19 @@ _clutter_context_create_pango_context (ClutterMainContext *self)
 {
   PangoContext *context;
 
+  if (self->font_map == NULL)
+    {
+      gdouble resolution;
+
+      self->font_map = COGL_PANGO_FONT_MAP (cogl_pango_font_map_new ());
+
+      resolution = clutter_backend_get_resolution (self->backend);
+      cogl_pango_font_map_set_resolution (self->font_map, resolution);
+
+      if (G_LIKELY (!clutter_disable_mipmap_text))
+        cogl_pango_font_map_set_use_mipmapping (self->font_map, TRUE);
+    }
+
   context = cogl_pango_font_map_create_context (self->font_map);
   update_pango_context (self->backend, context);
   pango_context_set_language (context, pango_language_get_default ());
@@ -723,6 +805,28 @@ clutter_main_level (void)
   return clutter_main_loop_level;
 }
 
+#ifdef CLUTTER_ENABLE_PROFILE
+static gint (*prev_poll) (GPollFD *ufds, guint nfsd, gint timeout_) = NULL;
+
+static gint
+timed_poll (GPollFD *ufds,
+            guint nfsd,
+            gint timeout_)
+{
+  gint ret;
+  CLUTTER_STATIC_TIMER (poll_timer,
+                        "Mainloop", /* parent */
+                        "poll (idle)",
+                        "The time spent idle in poll()",
+                        0 /* no application private data */);
+
+  CLUTTER_TIMER_START (_clutter_uprof_context, poll_timer);
+  ret = prev_poll (ufds, nfsd, timeout_);
+  CLUTTER_TIMER_STOP (_clutter_uprof_context, poll_timer);
+  return ret;
+}
+#endif
+
 /**
  * clutter_main:
  *
@@ -732,6 +836,14 @@ void
 clutter_main (void)
 {
   GMainLoop *loop;
+  CLUTTER_STATIC_TIMER (mainloop_timer,
+                        NULL, /* no parent */
+                        "Mainloop",
+                        "The time spent in the clutter mainloop",
+                        0 /* no application private data */);
+
+  if (clutter_main_loop_level == 0)
+    CLUTTER_TIMER_START (_clutter_uprof_context, mainloop_timer);
 
   /* Make sure there is a context */
   CLUTTER_CONTEXT ();
@@ -747,6 +859,14 @@ clutter_main (void)
 
   clutter_main_loop_level++;
 
+#ifdef CLUTTER_ENABLE_PROFILE
+  if (!prev_poll)
+    {
+      prev_poll = g_main_context_get_poll_func (NULL);
+      g_main_context_set_poll_func (NULL, timed_poll);
+    }
+#endif
+
   loop = g_main_loop_new (NULL, TRUE);
   main_loops = g_slist_prepend (main_loops, loop);
 
@@ -771,6 +891,9 @@ clutter_main (void)
   clutter_main_loop_level--;
 
   CLUTTER_MARK ();
+
+  if (clutter_main_loop_level == 0)
+    CLUTTER_TIMER_STOP (_clutter_uprof_context, mainloop_timer);
 }
 
 static void
@@ -1333,6 +1456,32 @@ clutter_arg_no_debug_cb (const char *key,
 }
 #endif /* CLUTTER_ENABLE_DEBUG */
 
+#ifdef CLUTTER_ENABLE_PROFILE
+static gboolean
+clutter_arg_profile_cb (const char *key,
+                        const char *value,
+                        gpointer    user_data)
+{
+  clutter_profile_flags |=
+    g_parse_debug_string (value,
+                          clutter_profile_keys,
+                          G_N_ELEMENTS (clutter_profile_keys));
+  return TRUE;
+}
+
+static gboolean
+clutter_arg_no_profile_cb (const char *key,
+                           const char *value,
+                           gpointer    user_data)
+{
+  clutter_profile_flags &=
+    ~g_parse_debug_string (value,
+                           clutter_profile_keys,
+                           G_N_ELEMENTS (clutter_profile_keys));
+  return TRUE;
+}
+#endif /* CLUTTER_ENABLE_PROFILE */
+
 GQuark
 clutter_init_error_quark (void)
 {
@@ -1343,8 +1492,6 @@ static ClutterInitError
 clutter_init_real (GError **error)
 {
   ClutterMainContext *ctx;
-  ClutterActor *stage;
-  gdouble resolution;
   ClutterBackend *backend;
 
   /* Note, creates backend if not already existing, though parse args will
@@ -1355,10 +1502,14 @@ clutter_init_real (GError **error)
 
   if (!ctx->options_parsed)
     {
-      g_set_error (error, CLUTTER_INIT_ERROR,
-		   CLUTTER_INIT_ERROR_INTERNAL,
-		   "When using clutter_get_option_group_without_init() "
-		   "you must parse options before calling clutter_init()");
+      if (error)
+        g_set_error (error, CLUTTER_INIT_ERROR,
+                     CLUTTER_INIT_ERROR_INTERNAL,
+                     "When using clutter_get_option_group_without_init() "
+		     "you must parse options before calling clutter_init()");
+      else
+        g_critical ("When using clutter_get_option_group_without_init() "
+		    "you must parse options before calling clutter_init()");
 
       return CLUTTER_INIT_ERROR_INTERNAL;
     }
@@ -1369,55 +1520,22 @@ clutter_init_real (GError **error)
   if (!_clutter_backend_post_parse (backend, error))
     return CLUTTER_INIT_ERROR_BACKEND;
 
-  /* Stage will give us a GL Context etc */
-  stage = clutter_stage_get_default ();
-  if (!stage)
-    {
-      if (error)
-        g_set_error (error, CLUTTER_INIT_ERROR,
-                     CLUTTER_INIT_ERROR_INTERNAL,
-                     "Unable to create the default stage");
-      else
-        g_critical ("Unable to create the default stage");
-
-      return CLUTTER_INIT_ERROR_INTERNAL;
-    }
-
-  clutter_stage_set_title (CLUTTER_STAGE (stage), g_get_prgname ());
-
-  clutter_actor_realize (stage);
-
-  if (!CLUTTER_ACTOR_IS_REALIZED (stage))
-    {
-      if (error)
-        g_set_error (error, CLUTTER_INIT_ERROR,
-                     CLUTTER_INIT_ERROR_INTERNAL,
-                     "Unable to realize the default stage");
-      else
-        g_critical ("Unable to realize the default stage");
-
-      return CLUTTER_INIT_ERROR_INTERNAL;
-    }
-
-  /* Now we can safely assume we have a valid GL context and can
-   * start issueing cogl commands
-  */
-  /* - will call to backend and cogl */
+  /* this will take care of initializing Cogl's state and
+   * query the GL machinery for features
+   */
   _clutter_feature_init ();
 
-  /*
-   * Resolution requires display to be open, so can only be queried after
-   * the post_parse hooks run.
-   *
-   * NB: cogl_pango requires a Cogl context.
-   */
-  ctx->font_map = COGL_PANGO_FONT_MAP (cogl_pango_font_map_new ());
+#ifdef CLUTTER_ENABLE_PROFILE
+    {
+      UProfContext *cogl_context;
+      cogl_context = uprof_find_context ("Cogl");
+      if (cogl_context)
+        uprof_context_link (_clutter_uprof_context, cogl_context);
+    }
 
-  resolution = clutter_backend_get_resolution (ctx->backend);
-  cogl_pango_font_map_set_resolution (ctx->font_map, resolution);
-
-  if (!clutter_disable_mipmap_text)
-    cogl_pango_font_map_set_use_mipmapping (ctx->font_map, TRUE);
+  if (clutter_profile_flags & CLUTTER_PROFILE_PICKING_ONLY)
+    _clutter_profile_suspend ();
+#endif
 
   clutter_text_direction = clutter_get_text_direction ();
 
@@ -1467,6 +1585,12 @@ static GOptionEntry clutter_args[] = {
   { "clutter-no-debug", 0, 0, G_OPTION_ARG_CALLBACK, clutter_arg_no_debug_cb,
     N_("Clutter debugging flags to unset"), "FLAGS" },
 #endif /* CLUTTER_ENABLE_DEBUG */
+#ifdef CLUTTER_ENABLE_PROFILE
+  { "clutter-profile", 0, 0, G_OPTION_ARG_CALLBACK, clutter_arg_profile_cb,
+    N_("Clutter profiling flags to set"), "FLAGS" },
+  { "clutter-no-profile", 0, 0, G_OPTION_ARG_CALLBACK, clutter_arg_no_profile_cb,
+    N_("Clutter profiling flags to unset"), "FLAGS" },
+#endif /* CLUTTER_ENABLE_PROFILE */
   { NULL, },
 };
 
@@ -1510,6 +1634,18 @@ pre_parse_hook (GOptionContext  *context,
     }
 #endif /* CLUTTER_ENABLE_DEBUG */
 
+#ifdef CLUTTER_ENABLE_PROFILE
+  env_string = g_getenv ("CLUTTER_PROFILE");
+  if (env_string != NULL)
+    {
+      clutter_profile_flags =
+        g_parse_debug_string (env_string,
+                              clutter_profile_keys,
+                              G_N_ELEMENTS (clutter_profile_keys));
+      env_string = NULL;
+    }
+#endif /* CLUTTER_ENABLE_PROFILE */
+
   env_string = g_getenv ("CLUTTER_SHOW_FPS");
   if (env_string)
     clutter_show_fps = TRUE;
@@ -1879,69 +2015,94 @@ event_click_count_generate (ClutterEvent *event)
   static guint32 previous_time          = 0;
   static gint    previous_button_number = -1;
 
+  ClutterInputDevice *device = NULL;
   ClutterBackend *backend;
-  guint           double_click_time;
-  guint           double_click_distance;
+  guint double_click_time;
+  guint double_click_distance;
 
-  backend = _clutter_context_get_default ()->backend;
+  backend = clutter_get_default_backend ();
   double_click_distance = clutter_backend_get_double_click_distance (backend);
   double_click_time = clutter_backend_get_double_click_time (backend);
 
-  if (event->button.device != NULL)
+  device = clutter_event_get_device (event);
+  if (device != NULL)
     {
-      click_count = event->button.device->click_count;
-      previous_x = event->button.device->previous_x;
-      previous_y = event->button.device->previous_y;
-      previous_time = event->button.device->previous_time;
-      previous_button_number = event->button.device->previous_button_number;
+      click_count = device->click_count;
+      previous_x = device->previous_x;
+      previous_y = device->previous_y;
+      previous_time = device->previous_time;
+      previous_button_number = device->previous_button_number;
+
+      CLUTTER_NOTE (EVENT,
+                    "Restoring previous click count:%d (device:%d, time:%u)",
+                    click_count,
+                    clutter_input_device_get_device_id (device),
+                    previous_time);
+    }
+  else
+    {
+      CLUTTER_NOTE (EVENT,
+                    "Restoring previous click count:%d (time:%u)",
+                    click_count,
+                    previous_time);
     }
 
-  switch (event->type)
+  switch (clutter_event_type (event))
     {
       case CLUTTER_BUTTON_PRESS:
-      case CLUTTER_SCROLL:
         /* check if we are in time and within distance to increment an
          * existing click count
          */
-        if (event->button.time < previous_time + double_click_time &&
+        if (event->button.button == previous_button_number &&
+            event->button.time < (previous_time + double_click_time) &&
             (ABS (event->button.x - previous_x) <= double_click_distance) &&
-            (ABS (event->button.y - previous_y) <= double_click_distance)
-            && event->button.button == previous_button_number)
+            (ABS (event->button.y - previous_y) <= double_click_distance))
           {
-            click_count ++;
+            CLUTTER_NOTE (EVENT, "Increase click count (button: %d, time: %u)",
+                          event->button.button,
+                          event->button.time);
+
+            click_count += 1;
           }
         else /* start a new click count*/
           {
-            click_count=1;
+            CLUTTER_NOTE (EVENT, "Reset click count (button: %d, time: %u)",
+                          event->button.button,
+                          event->button.time);
+
+            click_count = 1;
             previous_button_number = event->button.button;
           }
 
-        /* store time and position for this click for comparison with
-         * next event
-         */
+        previous_x = event->button.x;
+        previous_y = event->button.y;
         previous_time = event->button.time;
-        previous_x    = event->button.x;
-        previous_y    = event->button.y;
 
         /* fallthrough */
       case CLUTTER_BUTTON_RELEASE:
-        event->button.click_count=click_count;
+        event->button.click_count = click_count;
         break;
+
       default:
-        g_assert (NULL);
+        g_assert_not_reached ();
+        break;
     }
 
-  if (event->button.device != NULL)
+  if (event->type == CLUTTER_BUTTON_PRESS && device != NULL)
     {
-      event->button.device->click_count = click_count;
-      event->button.device->previous_x = previous_x;
-      event->button.device->previous_y = previous_y;
-      event->button.device->previous_time = previous_time;
-      event->button.device->previous_button_number = previous_button_number;
+      CLUTTER_NOTE (EVENT, "Storing click count: %d (device:%d, time:%u)",
+                    click_count,
+                    clutter_input_device_get_device_id (device),
+                    previous_time);
+
+      device->click_count = click_count;
+      device->previous_x = previous_x;
+      device->previous_y = previous_y;
+      device->previous_time = previous_time;
+      device->previous_button_number = previous_button_number;
     }
 }
 
-
 static inline void
 emit_event (ClutterEvent *event,
             gboolean      is_key_event)
@@ -1952,7 +2113,7 @@ emit_event (ClutterEvent *event,
   ClutterActor *actor;
   gint i = 0;
 
-  if (!event->any.source)
+  if (event->any.source == NULL)
     {
       CLUTTER_NOTE (EVENT, "No source set, discarding event");
       return;
@@ -2016,136 +2177,49 @@ static inline void
 emit_pointer_event (ClutterEvent       *event,
                     ClutterInputDevice *device)
 {
-  /* Using the global variable directly, since it has to be initialized
-   * at this point
-   */
-  ClutterMainContext *context = ClutterCntx;
+  ClutterMainContext *context = _clutter_context_get_default ();
 
-  if (G_UNLIKELY (context->pointer_grab_actor != NULL &&
-                  device == NULL))
-    {
-      /* global grab */
-      clutter_actor_event (context->pointer_grab_actor, event, FALSE);
-    }
-  else if (G_UNLIKELY (device != NULL &&
-                       device->pointer_grab_actor != NULL))
-    {
-      /* per device grab */
-      clutter_actor_event (device->pointer_grab_actor, event, FALSE);
-    }
-  else
+  if (context->pointer_grab_actor == NULL &&
+      (device == NULL || device->pointer_grab_actor == NULL))
     {
       /* no grab, time to capture and bubble */
       emit_event (event, FALSE);
     }
+  else
+    {
+      if (context->pointer_grab_actor != NULL)
+        {
+          /* global grab */
+          clutter_actor_event (context->pointer_grab_actor, event, FALSE);
+        }
+      else if (device != NULL && device->pointer_grab_actor != NULL)
+        {
+          /* per device grab */
+          clutter_actor_event (device->pointer_grab_actor, event, FALSE);
+        }
+    }
 }
 
 static inline void
 emit_keyboard_event (ClutterEvent *event)
 {
-  ClutterMainContext *context = ClutterCntx;
+  ClutterMainContext *context = _clutter_context_get_default ();
 
-  if (G_UNLIKELY (context->keyboard_grab_actor != NULL))
-    clutter_actor_event (context->keyboard_grab_actor, event, FALSE);
-  else
+  if (context->keyboard_grab_actor == NULL)
     emit_event (event, TRUE);
-}
-
-static void
-unset_motion_last_actor (ClutterActor *actor, ClutterInputDevice *dev)
-{
-  ClutterMainContext *context = ClutterCntx;
-
-  if (dev == NULL)
-    context->motion_last_actor = NULL;
   else
-    dev->motion_last_actor = NULL;
+    clutter_actor_event (context->keyboard_grab_actor, event, FALSE);
 }
 
-static void
-set_motion_last_actor (ClutterActor       *motion_current_actor,
-                       ClutterInputDevice *device)
+static gboolean
+is_off_stage (ClutterActor *stage,
+              gfloat        x,
+              gfloat        y)
 {
-  ClutterMainContext *context              = ClutterCntx;
-  ClutterActor       *last_actor           = context->motion_last_actor;
-
-  if (device != NULL)
-    last_actor = device->motion_last_actor;
-
-  if (last_actor && last_actor != motion_current_actor)
-    {
-      g_signal_handlers_disconnect_by_func
-                       (last_actor,
-                        G_CALLBACK (unset_motion_last_actor),
-                        device);
-    }
-
-  if (motion_current_actor && last_actor != motion_current_actor)
-    {
-      g_signal_connect (motion_current_actor, "destroy",
-                        G_CALLBACK (unset_motion_last_actor),
-                        device);
-    }
-
-  if (device != NULL)
-    device->motion_last_actor = motion_current_actor;
-  else
-    context->motion_last_actor = motion_current_actor;
-}
-
-static inline void
-generate_enter_leave_events (ClutterEvent *event)
-{
-  ClutterMainContext *context              = ClutterCntx;
-  ClutterActor       *motion_current_actor = event->motion.source;
-  ClutterActor       *last_actor           = context->motion_last_actor;
-  ClutterInputDevice *device               = clutter_event_get_device (event);
-
-  if (device != NULL)
-    last_actor = device->motion_last_actor;
-
-  if (last_actor != motion_current_actor)
-    {
-      if (motion_current_actor)
-        {
-          ClutterEvent cev;
-          gfloat x, y;
-
-          cev.crossing.device  = device;
-          clutter_event_get_coords (event, &x, &y);
-
-          if (context->motion_last_actor)
-            {
-              cev.crossing.type    = CLUTTER_LEAVE;
-              cev.crossing.time    = event->any.time;
-              cev.crossing.flags   = 0;
-              cev.crossing.x       = x;
-              cev.crossing.y       = y;
-              cev.crossing.source  = last_actor;
-              cev.crossing.stage   = event->any.stage;
-              cev.crossing.related = motion_current_actor;
-
-              emit_pointer_event (&cev, device);
-            }
-
-          cev.crossing.type    = CLUTTER_ENTER;
-          cev.crossing.time    = event->any.time;
-          cev.crossing.flags   = 0;
-          cev.crossing.x       = x;
-          cev.crossing.y       = y;
-          cev.crossing.source  = motion_current_actor;
-          cev.crossing.stage   = event->any.stage;
-
-          if (context->motion_last_actor)
-            cev.crossing.related = last_actor;
-          else
-            cev.crossing.related = NULL;
-
-          emit_pointer_event (&cev, device);
-        }
-    }
-
-  set_motion_last_actor (motion_current_actor, device);
+  return (x < 0 ||
+          y < 0 ||
+          x >= clutter_actor_get_width (stage) ||
+          y >= clutter_actor_get_height (stage));
 }
 
 /**
@@ -2177,7 +2251,9 @@ _clutter_process_event_details (ClutterActor        *stage,
                                 ClutterMainContext  *context,
                                 ClutterEvent        *event)
 {
-  ClutterInputDevice  *device = NULL;
+  ClutterInputDevice *device = NULL;
+
+  device = clutter_event_get_device (event);
 
   switch (event->type)
     {
@@ -2186,37 +2262,15 @@ _clutter_process_event_details (ClutterActor        *stage,
         break;
 
       case CLUTTER_LEAVE:
-        /* The source is set for generated events, not for events
-         * resulting from the cursor leaving the stage
-         */
-        if (event->any.source == NULL)
-          {
-            ClutterActor *last_actor = context->motion_last_actor;
-
-            if (event->crossing.device != NULL)
-              last_actor = event->crossing.device->motion_last_actor;
-
-            event->any.source = last_actor;
-
-            set_motion_last_actor (NULL, event->crossing.device);
-          }
-        /* flow through */
       case CLUTTER_ENTER:
-        emit_pointer_event (event, event->crossing.device);
+        emit_pointer_event (event, device);
         break;
 
       case CLUTTER_DESTROY_NOTIFY:
       case CLUTTER_DELETE:
         event->any.source = stage;
         /* the stage did not handle the event, so we just quit */
-        if (!clutter_stage_event (CLUTTER_STAGE (stage), event))
-          {
-            if (stage == clutter_stage_get_default())
-              clutter_main_quit ();
-            else
-              clutter_actor_destroy (stage);
-          }
-
+        clutter_stage_event (CLUTTER_STAGE (stage), event);
         break;
 
       case CLUTTER_KEY_PRESS:
@@ -2241,8 +2295,6 @@ _clutter_process_event_details (ClutterActor        *stage,
         break;
 
       case CLUTTER_MOTION:
-        device = event->motion.device;
-
         /* Only stage gets motion events if clutter_set_motion_events is TRUE,
          * and the event is not a synthetic event with source set.
          */
@@ -2275,8 +2327,7 @@ _clutter_process_event_details (ClutterActor        *stage,
             break;
           }
 
-        /* fallthrough */
-
+      /* fallthrough from motion */
       case CLUTTER_BUTTON_PRESS:
       case CLUTTER_BUTTON_RELEASE:
       case CLUTTER_SCROLL:
@@ -2291,10 +2342,14 @@ _clutter_process_event_details (ClutterActor        *stage,
            */
           if (event->any.source == NULL)
             {
-              /* Handle release off stage */
-              if ((x >= clutter_actor_get_width (stage) ||
-                   y >= clutter_actor_get_height (stage) ||
-                   x < 0 || y < 0))
+              /* emulate X11 the implicit soft grab; the implicit soft grab
+               * keeps relaying motion events when the stage is left with a
+               * pointer button pressed. since this is what happens when we
+               * disable per-actor motion events we need to maintain the same
+               * behaviour when the per-actor motion events are enabled as
+               * well
+               */
+              if (is_off_stage (stage, x, y))
                 {
                   if (event->type == CLUTTER_BUTTON_RELEASE)
                     {
@@ -2303,19 +2358,43 @@ _clutter_process_event_details (ClutterActor        *stage,
                                     x, y);
 
                       event->button.source = stage;
-                      emit_pointer_event (event, event->button.device);
+                      event->button.click_count = 1;
+
+                      emit_pointer_event (event, device);
                     }
+                  else if (event->type == CLUTTER_MOTION)
+                    {
+                      CLUTTER_NOTE (EVENT,
+                                    "Motion off stage received at %.2f, %2.f",
+                                    x, y);
+
+                      event->motion.source = stage;
+
+                      emit_pointer_event (event, device);
+                    }
+
                   break;
                 }
 
-              /* Map the event to a reactive actor */
-              actor = _clutter_do_pick (CLUTTER_STAGE (stage),
-                                        x, y,
-                                        CLUTTER_PICK_REACTIVE);
+              /* if the backend provides a device then we should
+               * already have everything we need to update it and
+               * get the actor underneath
+               */
+              if (device != NULL)
+                actor = _clutter_input_device_update (device);
+              else
+                {
+                  CLUTTER_NOTE (EVENT, "No device found: picking");
+
+                  actor = _clutter_do_pick (CLUTTER_STAGE (stage),
+                                            x, y,
+                                            CLUTTER_PICK_REACTIVE);
+                }
+
+              if (actor == NULL)
+                break;
 
               event->any.source = actor;
-              if (!actor)
-                break;
             }
           else
             {
@@ -2323,7 +2402,6 @@ _clutter_process_event_details (ClutterActor        *stage,
               actor = event->any.source;
             }
 
-
           /* FIXME: for an optimisation should check if there are
            * actually any reactive actors and avoid the pick all together
            * (signalling just the stage). Should be big help for gles.
@@ -2334,33 +2412,14 @@ _clutter_process_event_details (ClutterActor        *stage,
                         x, y,
                         actor);
 
-          /* Create, enter/leave events if needed */
-          generate_enter_leave_events (event);
-
-          if (event->type != CLUTTER_MOTION)
+          /* button presses and releases need a click count */
+          if (event->type == CLUTTER_BUTTON_PRESS ||
+              event->type == CLUTTER_BUTTON_RELEASE)
             {
               /* Generate click count */
               event_click_count_generate (event);
             }
 
-          if (device == NULL)
-            {
-              switch (event->type)
-                {
-                  case CLUTTER_BUTTON_PRESS:
-                  case CLUTTER_BUTTON_RELEASE:
-                    device = event->button.device;
-                    break;
-                  case CLUTTER_SCROLL:
-                    device = event->scroll.device;
-                    break;
-                  case CLUTTER_MOTION:
-                    /* already handled in the MOTION case of the switch */
-                  default:
-                    break;
-                }
-            }
-
           emit_pointer_event (event, device);
           break;
         }
@@ -2864,9 +2923,19 @@ clutter_get_font_flags (void)
 
 /**
  * clutter_get_input_device_for_id:
- * @id: a device id
+ * @id: the unique id for a device
  *
- * Retrieves the #ClutterInputDevice from its id.
+ * Retrieves the #ClutterInputDevice from its @id. This is a convenience
+ * wrapper for clutter_device_manager_get_device() and it is functionally
+ * equivalent to:
+ *
+ * |[
+ *   ClutterDeviceManager *manager;
+ *   ClutterInputDevice *device;
+ *
+ *   manager = clutter_device_manager_get_default ();
+ *   device = clutter_device_manager_get_device (manager, id);
+ * ]|
  *
  * Return value: (transfer none): a #ClutterInputDevice, or %NULL
  *
@@ -2875,23 +2944,11 @@ clutter_get_font_flags (void)
 ClutterInputDevice *
 clutter_get_input_device_for_id (gint id)
 {
-  GSList *item;
-  ClutterInputDevice *device = NULL;
-  ClutterMainContext  *context;
+  ClutterDeviceManager *manager;
 
-  context = _clutter_context_get_default ();
+  manager = clutter_device_manager_get_default ();
 
-  for (item = context->input_devices;
-       item != NULL;
-       item = item->next)
-  {
-    device = item->data;
-
-    if (device->id == id)
-      return device;
-  }
-
-  return NULL;
+  return clutter_device_manager_get_device (manager, id);
 }
 
 /**
