@@ -1,3 +1,41 @@
+/*
+ * Clutter.
+ *
+ * An OpenGL based 'interactive canvas' library.
+ *
+ * Copyright (C) 2008 OpenedHand
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Author: Emmanuele Bassi <ebassi@linux.intel.com>
+ */
+
+/**
+ * SECTION:clutter-stage-manager
+ * @short_description: Maintains the list of stages
+ *
+ * #ClutterStageManager is a singleton object, owned by Clutter, which
+ * maintains the list of currently active stages
+ *
+ * Every newly-created #ClutterStage will cause the emission of the
+ * #ClutterStageManager::stage-added signal; once a #ClutterStage has
+ * been destroyed, the #ClutterStageManager::stage-removed signal will
+ * be emitted
+ *
+ * #ClutterStageManager is available since Clutter 0.8
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -28,24 +66,6 @@ static ClutterStage *default_stage = NULL;
 
 G_DEFINE_TYPE (ClutterStageManager, clutter_stage_manager, G_TYPE_OBJECT);
 
-static void
-clutter_stage_manager_set_property (GObject      *gobject,
-                                    guint         prop_id,
-                                    const GValue *value,
-                                    GParamSpec   *pspec)
-{
-  switch (prop_id)
-    {
-    case PROP_DEFAULT_STAGE:
-      clutter_stage_manager_set_default_stage (CLUTTER_STAGE_MANAGER (gobject),
-                                               g_value_get_object (value));
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, prop_id, pspec);
-      break;
-    }
-}
-
 static void
 clutter_stage_manager_get_property (GObject    *gobject,
                                     guint       prop_id,
@@ -92,7 +112,6 @@ clutter_stage_manager_class_init (ClutterStageManagerClass *klass)
   GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
 
   gobject_class->dispose      = clutter_stage_manager_dispose;
-  gobject_class->set_property = clutter_stage_manager_set_property;
   gobject_class->get_property = clutter_stage_manager_get_property;
 
   /**
@@ -108,10 +127,10 @@ clutter_stage_manager_class_init (ClutterStageManagerClass *klass)
                                                         "Default Stage",
                                                         "The default stage",
                                                         CLUTTER_TYPE_STAGE,
-                                                        CLUTTER_PARAM_READWRITE));
+                                                        CLUTTER_PARAM_READABLE));
 
   /**
-   * ClutterStageManager:stage-added:
+   * ClutterStageManager::stage-added:
    * @stage_manager: the object which received the signal
    * @stage: the added stage
    *
@@ -153,7 +172,6 @@ clutter_stage_manager_class_init (ClutterStageManagerClass *klass)
 static void
 clutter_stage_manager_init (ClutterStageManager *stage_manager)
 {
-
 }
 
 /**
@@ -185,20 +203,13 @@ clutter_stage_manager_get_default (void)
  * Sets @stage as the default stage.
  *
  * Since: 0.8
+ *
+ * Deprecated: 1.2: Calling this function has no effect
  */
 void
 clutter_stage_manager_set_default_stage (ClutterStageManager *stage_manager,
                                          ClutterStage        *stage)
 {
-  g_return_if_fail (CLUTTER_IS_STAGE_MANAGER (stage_manager));
-  g_return_if_fail (CLUTTER_IS_STAGE (stage));
-
-  if (!g_slist_find (stage_manager->stages, stage))
-    _clutter_stage_manager_add_stage (stage_manager, stage);
-
-  default_stage = stage;
-
-  g_object_notify (G_OBJECT (stage_manager), "default-stage");
 }
 
 /**
@@ -219,13 +230,14 @@ clutter_stage_manager_get_default_stage (ClutterStageManager *stage_manager)
 }
 
 /**
- * clutter_stage_manager_list_stage:
+ * clutter_stage_manager_list_stages:
  * @stage_manager: a #ClutterStageManager
  *
  * Lists all currently used stages.
  *
- * Return value: (transfer container) (element-type ClutterStage): a newly allocated list
- *   of #ClutterStage objects. Use g_slist_free() to deallocate it when done.
+ * Return value: (transfer container) (element-type ClutterStage): a newly
+ *   allocated list of #ClutterStage objects. Use g_slist_free() to
+ *   deallocate it when done.
  *
  * Since: 0.8
  */
@@ -235,6 +247,25 @@ clutter_stage_manager_list_stages (ClutterStageManager *stage_manager)
   return g_slist_copy (stage_manager->stages);
 }
 
+/**
+ * clutter_stage_manager_peek_stages:
+ * @stage_manager: a #ClutterStageManager
+ *
+ * Lists all currently used stages.
+ *
+ * Return value: (transfer none) (element-type ClutterStage): a pointer
+ *   to the internal list of #ClutterStage objects. The returned list
+ *   is owned by the #ClutterStageManager and should never be modified
+ *   or freed
+ *
+ * Since: 1.0
+ */
+const GSList *
+clutter_stage_manager_peek_stages (ClutterStageManager *stage_manager)
+{
+  return stage_manager->stages;
+}
+
 void
 _clutter_stage_manager_add_stage (ClutterStageManager *stage_manager,
                                   ClutterStage        *stage)
@@ -250,7 +281,7 @@ _clutter_stage_manager_add_stage (ClutterStageManager *stage_manager,
 
   stage_manager->stages = g_slist_append (stage_manager->stages, stage);
 
-  if (!default_stage)
+  if (G_UNLIKELY (default_stage == NULL))
     {
       default_stage = stage;
 
