@@ -16,9 +16,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
- *
- *
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Authors:
  *  Neil Roberts   <neil@linux.intel.com>
@@ -67,8 +67,8 @@ struct _CoglAtlas
 {
   CoglAtlasNode *root;
 
-  unsigned int space_remaining;
-  unsigned int n_rectangles;
+  guint space_remaining;
+  guint n_rectangles;
 
   GDestroyNotify value_destroy_func;
 };
@@ -119,8 +119,7 @@ _cogl_atlas_node_free (CoglAtlasNode *node)
 }
 
 CoglAtlas *
-_cogl_atlas_new (unsigned int width,
-                 unsigned int height,
+_cogl_atlas_new (guint width, guint height,
                  GDestroyNotify value_destroy_func)
 {
   CoglAtlas *atlas = g_new (CoglAtlas, 1);
@@ -167,7 +166,7 @@ _cogl_atlas_stack_pop (CoglAtlasStackEntry *stack)
 
 static CoglAtlasNode *
 _cogl_atlas_node_split_horizontally (CoglAtlasNode *node,
-                                     unsigned int left_width)
+                                     guint left_width)
 {
   /* Splits the node horizontally (according to emacs' definition, not
      vim) by converting it to a branch and adding two new leaf
@@ -205,7 +204,7 @@ _cogl_atlas_node_split_horizontally (CoglAtlasNode *node,
 
 static CoglAtlasNode *
 _cogl_atlas_node_split_vertically (CoglAtlasNode *node,
-                                   unsigned int top_height)
+                                   guint top_height)
 {
   /* Splits the node vertically (according to emacs' definition, not
      vim) by converting it to a branch and adding two new leaf
@@ -243,8 +242,7 @@ _cogl_atlas_node_split_vertically (CoglAtlasNode *node,
 
 gboolean
 _cogl_atlas_add_rectangle (CoglAtlas *atlas,
-                           unsigned int width,
-                           unsigned int height,
+                           guint width, guint height,
                            gpointer data,
                            CoglAtlasRectangle *rectangle)
 {
@@ -413,25 +411,25 @@ _cogl_atlas_remove_rectangle (CoglAtlas *atlas,
 #endif
 }
 
-unsigned int
+guint
 _cogl_atlas_get_width (CoglAtlas *atlas)
 {
   return atlas->root->rectangle.width;
 }
 
-unsigned int
+guint
 _cogl_atlas_get_height (CoglAtlas *atlas)
 {
   return atlas->root->rectangle.height;
 }
 
-unsigned int
+guint
 _cogl_atlas_get_remaining_space (CoglAtlas *atlas)
 {
   return atlas->space_remaining;
 }
 
-unsigned int
+guint
 _cogl_atlas_get_n_rectangles (CoglAtlas *atlas)
 {
   return atlas->n_rectangles;
