@@ -61,7 +61,7 @@ typedef struct _TextureSlicedQuadState
 
 typedef struct _TextureSlicedPolygonState
 {
-  CoglTextureVertex *vertices;
+  const CoglTextureVertex *vertices;
   int n_vertices;
   int stride;
 } TextureSlicedPolygonState;
@@ -71,8 +71,8 @@ static void
 log_quad_sub_textures_cb (CoglHandle texture_handle,
                           GLuint gl_handle,
                           GLenum gl_target,
-                          float *subtexture_coords,
-                          float *virtual_coords,
+                          const float *subtexture_coords,
+                          const float *virtual_coords,
                           void *user_data)
 {
   TextureSlicedQuadState *state = user_data;
@@ -249,6 +249,9 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
       const float       *in_tex_coords;
       float             *out_tex_coords;
       float              default_tex_coords[4] = {0.0, 0.0, 1.0, 1.0};
+      gboolean           need_repeat = FALSE;
+      gint               coord_num;
+      GLenum             wrap_mode;
 
       tex_handle = cogl_material_layer_get_texture (layer);
 
@@ -257,27 +260,41 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
       if (tex_handle == COGL_INVALID_HANDLE)
         continue;
 
-      in_tex_coords = &user_tex_coords[i * 4];
+      /* If the user didn't supply texture coordinates for this layer
+         then use the default coords */
+      if (i >= user_tex_coords_len / 4)
+        in_tex_coords = default_tex_coords;
+      else
+        in_tex_coords = &user_tex_coords[i * 4];
+
       out_tex_coords = &final_tex_coords[i * 4];
 
+      memcpy (out_tex_coords, in_tex_coords, sizeof (GLfloat) * 4);
+
+      /* Convert the texture coordinates to GL. We also work out
+         whether any of the texture coordinates are outside the range
+         [0.0,1.0]. We need to do this after calling
+         transform_coords_to_gl in case the texture backend is munging
+         the coordinates (such as in the sub texture backend). This
+         should be safe to call because we know that the texture only
+         has one slice. */
+      if (!_cogl_texture_transform_quad_coords_to_gl (tex_handle,
+                                                      out_tex_coords))
+        /* If the backend can't support these coordinates then bail out */
+        return FALSE;
+      for (coord_num = 0; coord_num < 4; coord_num++)
+        if (out_tex_coords[coord_num] < 0.0f ||
+            out_tex_coords[coord_num] > 1.0f)
+          need_repeat = TRUE;
 
       /* If the texture has waste or we are using GL_TEXTURE_RECT we
-       * can't handle texture repeating so we check that the texture
-       * coords lie in the range [0,1].
-       *
-       * NB: We already know that the texture isn't sliced so we can assume
-       * that the default coords (0,0) and (1,1) would only reference a single
-       * GL texture.
+       * can't handle texture repeating so we can't use the layer if
+       * repeating is required.
        *
        * NB: We already know that no texture matrix is being used if the
        * texture doesn't support hardware repeat.
        */
-      if (!_cogl_texture_can_hardware_repeat (tex_handle)
-           && i < user_tex_coords_len / 4
-           && (in_tex_coords[0] < 0 || in_tex_coords[0] > 1.0
-               || in_tex_coords[1] < 0 || in_tex_coords[1] > 1.0
-               || in_tex_coords[2] < 0 || in_tex_coords[2] > 1.0
-               || in_tex_coords[3] < 0 || in_tex_coords[3] > 1.0))
+      if (!_cogl_texture_can_hardware_repeat (tex_handle) && need_repeat)
         {
           if (i == 0)
             {
@@ -315,45 +332,15 @@ _cogl_multitexture_quad_single_primitive (float        x_1,
             }
         }
 
-
-      /*
-       * Setup the texture unit...
-       */
-
-      /* NB: The user might not have supplied texture coordinates for all
-       * layers... */
-      if (i < (user_tex_coords_len / 4))
-        {
-          GLenum wrap_mode;
-
-          /* If the texture coords are all in the range [0,1] then we want to
-             clamp the coords to the edge otherwise it can pull in edge pixels
-             from the wrong side when scaled */
-          if (in_tex_coords[0] >= 0 && in_tex_coords[0] <= 1.0
-              && in_tex_coords[1] >= 0 && in_tex_coords[1] <= 1.0
-              && in_tex_coords[2] >= 0 && in_tex_coords[2] <= 1.0
-              && in_tex_coords[3] >= 0 && in_tex_coords[3] <= 1.0)
-            wrap_mode = GL_CLAMP_TO_EDGE;
-          else
-            wrap_mode = GL_REPEAT;
-
-          memcpy (out_tex_coords, in_tex_coords, sizeof (GLfloat) * 4);
-
-          _cogl_texture_set_wrap_mode_parameter (tex_handle, wrap_mode);
-        }
+      /* If we're not repeating then we want to clamp the coords
+         to the edge otherwise it can pull in edge pixels from the
+         wrong side when scaled */
+      if (need_repeat)
+        wrap_mode = GL_REPEAT;
       else
-        {
-          memcpy (out_tex_coords, default_tex_coords, sizeof (GLfloat) * 4);
+        wrap_mode = GL_CLAMP_TO_EDGE;
 
-          _cogl_texture_set_wrap_mode_parameter (tex_handle, GL_CLAMP_TO_EDGE);
-        }
-
-      _cogl_texture_transform_coords_to_gl (tex_handle,
-                                            &out_tex_coords[0],
-                                            &out_tex_coords[1]);
-      _cogl_texture_transform_coords_to_gl (tex_handle,
-                                            &out_tex_coords[2],
-                                            &out_tex_coords[3]);
+      _cogl_texture_set_wrap_mode_parameter (tex_handle, wrap_mode);
     }
 
   _cogl_journal_log_quad (x_1,
@@ -414,6 +401,12 @@ _cogl_rectangles_with_multitexture_coords (
 	  != COGL_MATERIAL_LAYER_TYPE_TEXTURE)
 	continue;
 
+      /* We need to ensure the mipmaps are ready before deciding
+         anything else about the texture because it could become
+         something completely different if it needs to be migrated out
+         of the atlas */
+      _cogl_material_layer_ensure_mipmaps (layer);
+
       tex_handle = cogl_material_layer_get_texture (layer);
 
       /* COGL_INVALID_HANDLE textures are handled by
@@ -651,11 +644,11 @@ cogl_rectangle (float x_1,
 
 void
 draw_polygon_sub_texture_cb (CoglHandle tex_handle,
-                       GLuint gl_handle,
-                       GLenum gl_target,
-                       float *subtexture_coords,
-                       float *virtual_coords,
-                       void *user_data)
+                             GLuint gl_handle,
+                             GLenum gl_target,
+                             const float *subtexture_coords,
+                             const float *virtual_coords,
+                             void *user_data)
 {
   TextureSlicedPolygonState *state = user_data;
   GLfloat *v;
@@ -709,7 +702,7 @@ draw_polygon_sub_texture_cb (CoglHandle tex_handle,
 
 /* handles 2d-sliced textures with > 1 slice */
 static void
-_cogl_texture_polygon_multiple_primitives (CoglTextureVertex *vertices,
+_cogl_texture_polygon_multiple_primitives (const CoglTextureVertex *vertices,
                                            unsigned int n_vertices,
                                            unsigned int stride,
                                            gboolean use_color)
@@ -762,7 +755,7 @@ _cogl_texture_polygon_multiple_primitives (CoglTextureVertex *vertices,
 }
 
 static void
-_cogl_multitexture_polygon_single_primitive (CoglTextureVertex *vertices,
+_cogl_multitexture_polygon_single_primitive (const CoglTextureVertex *vertices,
                                              guint n_vertices,
                                              guint n_layers,
                                              guint stride,
@@ -841,14 +834,13 @@ _cogl_multitexture_polygon_single_primitive (CoglTextureVertex *vertices,
 }
 
 void
-cogl_polygon (CoglTextureVertex *vertices,
-              guint              n_vertices,
-	      gboolean           use_color)
+cogl_polygon (const CoglTextureVertex *vertices,
+              guint                    n_vertices,
+	      gboolean                 use_color)
 {
   CoglHandle           material;
-  const GList         *layers;
+  const GList         *layers, *tmp;
   int                  n_layers;
-  GList               *tmp;
   gboolean	       use_sliced_polygon_fallback = FALSE;
   guint32              fallback_layers = 0;
   int                  i;
@@ -871,16 +863,21 @@ cogl_polygon (CoglTextureVertex *vertices,
   layers = cogl_material_get_layers (ctx->source_material);
   n_layers = g_list_length ((GList *)layers);
 
-  for (tmp = (GList *)layers, i = 0; tmp != NULL; tmp = tmp->next, i++)
+  for (tmp = layers, i = 0; tmp != NULL; tmp = tmp->next, i++)
     {
-      CoglHandle   layer = (CoglHandle)tmp->data;
-      CoglHandle   tex_handle = cogl_material_layer_get_texture (layer);
+      CoglHandle layer = tmp->data;
+      CoglHandle tex_handle = cogl_material_layer_get_texture (layer);
 
       /* COGL_INVALID_HANDLE textures will be handled in
        * _cogl_material_flush_layers_gl_state */
       if (tex_handle == COGL_INVALID_HANDLE)
         continue;
 
+      /* Give the texture a chance to know that we're rendering
+         non-quad shaped primitives. If the texture is in an atlas it
+         will be migrated */
+      _cogl_texture_ensure_non_quad_rendering (tex_handle);
+
       if (i == 0 && cogl_texture_is_sliced (tex_handle))
         {
 #if defined (HAVE_COGL_GLES) || defined (HAVE_COGL_GLES2)
@@ -904,6 +901,7 @@ cogl_polygon (CoglTextureVertex *vertices,
                   warning_seen = TRUE;
                 }
             }
+
           use_sliced_polygon_fallback = TRUE;
           n_layers = 1;
 
@@ -915,7 +913,7 @@ cogl_polygon (CoglTextureVertex *vertices,
                 {
                   g_warning ("cogl_texture_polygon does not work for sliced textures "
                              "when the minification and magnification filters are not "
-                             "CGL_NEAREST");
+                             "COGL_MATERIAL_FILTER_NEAREST");
                   warning_seen = TRUE;
                 }
               return;
@@ -990,9 +988,10 @@ cogl_polygon (CoglTextureVertex *vertices,
                              /* NB: [X,Y,Z,TX,TY...,R,G,B,A,...] */
                              v + 3 + 2 * i));
     }
-  prev_n_texcoord_arrays_enabled =
-    ctx->n_texcoord_arrays_enabled;
+
+  prev_n_texcoord_arrays_enabled = ctx->n_texcoord_arrays_enabled;
   ctx->n_texcoord_arrays_enabled = n_layers;
+
   for (; i < prev_n_texcoord_arrays_enabled; i++)
     {
       GE (glClientActiveTexture (GL_TEXTURE0 + i));
@@ -1497,7 +1496,14 @@ _cogl_path_fill_nodes (void)
                       bounds_x + bounds_w, bounds_y + bounds_h);
 
       /* The stencil buffer now contains garbage so the clip area needs to
-         be rebuilt */
+       * be rebuilt.
+       *
+       * NB: We only ever try and update the clip state during
+       * _cogl_journal_init (when we flush the framebuffer state) which is
+       * only called when the journal first gets something logged in it; so
+       * we call cogl_flush() to emtpy the journal.
+       */
+      cogl_flush ();
       _cogl_clip_stack_state_dirty (clip_state);
     }
   else
