@@ -150,7 +150,9 @@ main (int argc, char **argv)
 
   TEST_CONFORM_SIMPLE ("/binding-pool", test_binding_pool);
 
+#if 0
   TEST_CONFORM_SIMPLE ("/actor", test_anchors);
+#endif
   TEST_CONFORM_SIMPLE ("/actor", test_actor_destruction);
 
   TEST_CONFORM_SIMPLE ("/model", test_list_model_populate);
@@ -160,6 +162,7 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/color", test_color_from_string);
   TEST_CONFORM_SIMPLE ("/color", test_color_to_string);
   TEST_CONFORM_SIMPLE ("/color", test_color_hls_roundtrip);
+  TEST_CONFORM_SIMPLE ("/color", test_color_operators);
 
   TEST_CONFORM_SIMPLE ("/units", test_units_constructors);
   TEST_CONFORM_SIMPLE ("/units", test_units_string);
@@ -176,6 +179,10 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/script", test_script_object_property);
   TEST_CONFORM_SIMPLE ("/script", test_script_animation);
   TEST_CONFORM_SIMPLE ("/script", test_script_named_object);
+  TEST_CONFORM_SIMPLE ("/script", test_animator_base);
+  TEST_CONFORM_SIMPLE ("/script", test_animator_properties);
+
+  TEST_CONFORM_SIMPLE ("/behaviours", test_behaviours);
 
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_fixed);
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_backface_culling);
@@ -185,6 +192,10 @@ main (int argc, char **argv)
   TEST_CONFORM_SIMPLE ("/cogl", test_cogl_readpixels);
 
   TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_npot_texture);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_multitexture);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_texture_mipmaps);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_sub_texture);
+  TEST_CONFORM_SIMPLE ("/cogl/texture", test_cogl_pixel_buffer);
 
   TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_contiguous);
   TEST_CONFORM_SIMPLE ("/cogl/vertex-buffer", test_cogl_vertex_buffer_interleved);
